unit FBDEUtil;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, DB, UtilWrap, BDE, DBTables, Buttons, PlugMan,
  Util, DBUtil;

type
  TFormBDEUtil = class(TPlugInForm)
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    FieldsLB: TLabel;
    RecSizeLB: TLabel;
    IndexLB: TLabel;
    ValidLB: TLabel;
    RefLB: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    RestructLB: TLabel;
    AuxPassLB: TLabel;
    CodePageLB: TLabel;
    BlockSizeLB: TLabel;
    TabLvlLB: TLabel;
    GroupBox3: TGroupBox;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;
    Label27: TLabel;
    Label28: TLabel;
    MessageLB: TLabel;
    PBHeader: TProgressBar;
    PBIndexes: TProgressBar;
    PBData: TProgressBar;
    PBRebuild: TProgressBar;
    EditTable: TEdit;
    BtnImpac: TSpeedButton;
    BtnRegen: TSpeedButton;
    BtnVerify: TSpeedButton;
    BtnRebuild: TSpeedButton;
    BtnSelTab: TButton;
    OpenDialog: TOpenDialog;
    Database: TDatabase;
    Table: TTable;
    TableVerRes: TTable;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnVerifyClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure BtnImpacClick(Sender: TObject);
    procedure BtnRegenClick(Sender: TObject);
    procedure BtnRebuildClick(Sender: TObject);
    procedure BtnSelTabClick(Sender: TObject);
  private
    UWrap: TUtilWrapper;
    CbInfo: TUVerifyCallback;
    TUProps: CURProps;
    vhTSes: hTUSes;
    TableDesc: CRTblDesc;
    function GetTCursorProps(szTable: String): Boolean;
    procedure RegisterCallBack;
    procedure UnRegisterCallBack;
    procedure ClearBars;
    procedure ClearLabels;
    procedure SetTableInfo;
  public
    Procedure Init(Linker: TPlugInLinker); Override;
    Function PackTable(DB: TDataBase; Table: TTable; Var Mes: String): Boolean;
    Function Restructure(DB: TDataBase; Table: TTable; NewName: String; Var Mes: String): Boolean;
    Function RegenIndex(DB: TDataBase; Table: TTable; Var Mes: String): Boolean;
  end;

var
  FormBDEUtil: TFormBDEUtil;

implementation

{$R *.DFM}

function GenProgressCallBack(ecbType: CBType; Data: LongInt; pcbInfo: Pointer): CBRType; stdcall;
Var
  CBInfo: TUVerifyCallBack;
Begin
  CBInfo := TUVerifyCallBack(pcbInfo^);
  If ecbType = cbGENPROGRESS then
  Begin
    If CBInfo.percentdone > 1000 Then
      FormBDEUtil.MessageLB.Caption := CBInfo.TableName
    Else
    Case CBInfo.Process Of
     TUVerifyHeader: FormBDEUtil.PBHeader.Position := CBInfo.percentdone;
     TUVerifyIndex:  FormBDEUtil.PBIndexes.Position := CBInfo.percentdone;
     TUVerifyData:   FormBDEUtil.PBData.Position := CBInfo.percentdone;
     TURebuild:      FormBDEUtil.PBRebuild.Position := CBInfo.percentdone;
    End;
    Application.ProcessMessages;
  End;

  Result := cbrUSEDEF;
end;

procedure TFormBDEUtil.FormCreate(Sender: TObject);
Var
  Path: String;
begin
  Path := SlashSep(ExtractFilePath(ParamStr(0)), 'PlugIns\TUtil32.DLL');
  UWrap := TUtilWrapper.Create(Path);
  Check(UWrap.UInit(vhtSes));
end;

procedure TFormBDEUtil.FormDestroy(Sender: TObject);
begin
  Check(UWrap.UExit(vhtSes));
  UWrap.Free;
end;

function TFormBDEUtil.GetTCursorProps(szTable: String): Boolean;
begin
  if UWrap.UFillCURProps(vHtSes, PChar(szTable), TUProps) = DBIERR_NONE then
    Result := True
  else Result := False;
end;

procedure TFormBDEUtil.RegisterCallback;
begin
 Check(DbiRegisterCallBack(nil, cbGENPROGRESS, 0,
            sizeof(TUVerifyCallBack), @CbInfo, GenProgressCallback));
end;

procedure TFormBDEUtil.UnRegisterCallback;
begin
  Check(DbiRegisterCallBack(nil, cbGENPROGRESS, 0,
           sizeof(TUVerifyCallBack), @CbInfo, nil));
end;

Function TFormBDEUtil.PackTable(DB: TDataBase; Table: TTable; Var Mes: String): Boolean;
begin
  FillChar(TableDesc, SizeOf(CRTblDesc), 0);
  StrPCopy(TableDesc.szTblName, Table.TableName);
  TableDesc.szTblType := szPARADOX;
  TableDesc.bPack := True;
  Result := Restructure(DB, Table, '', Mes);
End;

Function TFormBDEUtil.Restructure(DB: TDataBase; Table: TTable; NewName: String; Var Mes: String): Boolean;
Var
  BdeResult : DbiResult;
  SaveAs: PChar;
begin
  Table.Close;

  If NewName = '' Then SaveAs := Nil
  Else SaveAs := PChar(NewName);

  RegisterCallBack;
  Try
    BdeResult := DbiDoRestructure(DB.Handle, 1, @TableDesc, SaveAs, Nil, Nil, False);
  Finally
    UnRegisterCallBack;
  End;

  Result := (BdeResult = DBIERR_NONE);
  If Not Result Then Mes := 'Ristrutturazione Fallita!'
  Else Mes := 'Ristrutturazione Eseguita!';
End;

Function TFormBDEUtil.RegenIndex(DB: TDataBase; Table: TTable; Var Mes: String): Boolean;
Var
  BdeResult : DbiResult;
begin
  Result := False;
  Table.Close;
  Table.Exclusive := True;
  Try
    Table.Open;
  Except
  End;
  If Table.Active Then
  Begin
    RegisterCallBack;
    Try
      BdeResult := DbiRegenIndexes(Table.Handle);
    Finally
      UnRegisterCallBack;
    End;

    Case BdeResult of
      DBIERR_NONE           : Mes := 'Rigenerazione completata!';
      DBIERR_INVALIDHNDL    : Mes := 'Tabella non valida!';
      DBIERR_NEEDEXCLACCESS : Mes := 'Accesso negato!';
      DBIERR_NOTSUPPORTED   : Mes := 'Operazione non supportata!';
    else
     Mes := 'Errore inatteso!';
    end;

    Table.Close;
    Result := (BdeResult = DBIERR_NONE);
  End
  Else Mes := 'La tabella  in uso!';
  Table.Exclusive := False;
End;


Procedure TFormBDEUtil.Init(Linker: TPlugInLinker);
Begin
  Inherited;
  //
End;

procedure TFormBDEUtil.ClearBars;
begin
  MessageLB.Caption := '';
  PBHeader.Position := 0;
  PBIndexes.Position := 0;
  PBData.Position := 0;
  PBRebuild.Position := 0;
end;

procedure TFormBDEUtil.ClearLabels;
begin
  FieldsLB.Caption := '0';
  RecSizeLB.Caption := '0';
  IndexLB.Caption := '0';
  ValidLB.Caption := '0';
  RefLB.Caption := '0';
  RestructLB.Caption := '0';
  AuxPassLB.Caption := '0';
  CodePageLB.Caption := '0';
  BlockSizeLB.Caption := '0';
  TabLvlLB.Caption := '0';
end;

procedure TFormBDEUtil.SetTableInfo;
begin
  ClearLabels;
  If GetTCursorProps(Table.TableName) Then
    With TUProps Do
    Begin
      FieldsLB.Caption := IntToStr(iFields);
      RecSizeLB.Caption := IntToStr(iRecBufSize);
      IndexLB.Caption := IntToStr(iIndexes);
      ValidLB.Caption := InttoStr(iValChecks);
      RefLB.Caption := IntToStr(iRefIntChecks);
      RestructLB.Caption := IntToStr(iRestrVersion);
      AuxPassLB.Caption := IntToStr(iPasswords);
      CodePageLB.Caption := IntToStr(iCodePage);
      BlockSizeLB.Caption := IntToStr(iBlockSize);
      TabLvlLB.Caption := IntToStr(iTblLevel);
    End;
end;

procedure TFormBDEUtil.FormShow(Sender: TObject);
begin
  ClearBars;
end;

procedure TFormBDEUtil.BtnVerifyClick(Sender: TObject);
var
  Msg: Integer;
  szTable: String;
begin
  Screen.Cursor := crHourGlass;
  Table.Close;
  try
    ClearBars;
    Check(UWrap.UExit(vHtSes));
    Check(UWrap.UInit(vHtSes));
    szTable := EditTable.Text; //Table.TableName;
    RegisterCallBack;
    try
      if UWrap.UVerifyTable(vHtSes, PChar(szTable), szPARADOX, 'VERIFY.DB',
           nil, 0, Msg) = DBIERR_NONE then
      begin
        case Msg of
          0: MessageLB.Caption := 'Verifica Eseguita. La Tabella non ha errori.';
          1: MessageLB.Caption := 'Verifica Eseguita. Verifica completata.';
          2: MessageLB.Caption := 'Verifica Eseguita. La Verifica non pu essere completata.';
          3: MessageLB.Caption := 'Verifica Eseguita. La Tabella dovr essere ricostruita manualmente';
          4: MessageLB.Caption := 'Verifica Eseguita. La Tabella non pu essere ricostruita';
        else
          MessageLB.Caption := 'Verifica Fallita!';
        end;
      end;
    finally
      UnRegisterCallBack;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormBDEUtil.BtnRebuildClick(Sender: TObject);
var
  iFld, iIdx, iSec, iVal, iRI, iOptP, iOptD: word;
  szTable: String;
  rslt: DBIResult;
  Msg: Integer;
  TblDesc: CRTBlDesc;
  ErrorBuf: Array[0..255] Of Char;
begin
  Screen.Cursor := crHourGlass;
  Table.Close;
  try
    ClearBars;
    Check(UWrap.UExit(vHtSes));
    Check(UWrap.UInit(vHtSes));
    szTable := EditTable.Text; //Table.TableName;
    RegisterCallBack;
    try
      Check(UWrap.UVerifyTable(vHtSes, PChar(szTable), szPARADOX, 'VERIFY.DB',
           nil, 0, Msg));
      rslt := UWrap.UGetCRTblDescCount(vhTSes, PChar(szTable), iFld,
            iIdx, iSec, iVal, iRI, iOptP, iOptD);
      if rslt = DBIERR_NONE then
      begin
        FillChar(TblDesc, SizeOf(CRTBlDesc), 0);
        StrPCopy(TblDesc.szTblName, szTable);
        TblDesc.szTblType := szParadox;
        TblDesc.szErrTblName := 'Rebuild.DB';

        TblDesc.iFldCount := iFld;
        GetMem(TblDesc.pFldDesc, (iFld * SizeOf(FldDesc)));

        TblDesc.iIdxCount := iIdx;
        GetMem(TblDesc.pIdxDesc, (iIdx * SizeOf(IdxDesc)));

        TblDesc.iSecRecCount := iSec;
        GetMem(TblDesc.pSecDesc, (iSec * SizeOf(SecDesc)));

        TblDesc.iValChkCount := iVal;
        GetMem(TblDesc.pvchkDesc, (iVal * SizeOf(VCHKDesc)));

        TblDesc.iRintCount := iRI;
        GetMem(TblDesc.printDesc, (iRI * SizeOf(RINTDesc)));

        TblDesc.iOptParams := iOptP;
        GetMem(TblDesc.pfldOptParams, (iOptP * sizeOf(FLDDesc)));

        GetMem(TblDesc.pOptData, (iOptD * DBIMAXSCFLDLEN));
        Try
          rslt := UWrap.UFillCRTblDesc(vhTSes, @TblDesc, PChar(szTable), nil);
          If rslt = DBIERR_NONE then
          Begin
            rslt := UWrap.URebuildTable(vhTSes, PChar(szTable), szPARADOX,
                      'BACKUP.DB', 'KEYVIOL.DB', 'PROBLEM.DB', @TblDesc);
            If rslt = DBIERR_NONE then
              MessageLB.Caption := 'Ricostruzione Eseguita.'
            Else
            Begin
              UWrap.UGetErrorString(rslt, ErrorBuf);
              MessageLB.Caption := 'Ricostruzione Fallita. Errore: ' + StrPas(ErrorBuf);
            End;
          End
          Else MessageDlg('Errore di riempimento della struttura', mtError, [mbok], 0);
        Finally
          FreeMem(TblDesc.pFldDesc, (iFld * SizeOf(FldDesc)));
          FreeMem(TblDesc.pIdxDesc, (iIdx * SizeOf(IdxDesc)));
          FreeMem(TblDesc.pSecDesc, (iSec * SizeOf(SecDesc)));
          FreeMem(TblDesc.pvchkDesc, (iVal * SizeOf(VCHKDesc)));
          FreeMem(TblDesc.printDesc, (iRI * SizeOf(RINTDesc)));
          FreeMem(TblDesc.pfldOptParams, (iOptP * sizeOf(FLDDesc)));
          FreeMem(TblDesc.pOptData, (iOptD * DBIMAXSCFLDLEN));
        end;
      end;
    finally
      UnRegisterCallBack;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TFormBDEUtil.BtnImpacClick(Sender: TObject);
Var
  Mes: String;
begin
  Application.ProcessMessages;
  Screen.Cursor := crHourGlass;
  Try
    Table.Open;
    PackTable(Database, Table, Mes);
    Table.Close;
  Except
    Mes := 'Impossibile Eseguire l''operazione!';
  End;
  MessageLB.Caption := Mes;
  Screen.Cursor := crDefault;
end;

procedure TFormBDEUtil.BtnRegenClick(Sender: TObject);
Var
  Mes: String;
begin
  Application.ProcessMessages;
  Screen.Cursor := crHourGlass;
  Try
    Table.Open;
    RegenIndex(Database, Table, Mes);
    Table.Close;
  Except
    Mes := 'Impossibile Eseguire l''operazione!';
  End;
  MessageLB.Caption := Mes;
  Screen.Cursor := crDefault;
end;

procedure TFormBDEUtil.BtnSelTabClick(Sender: TObject);
begin
  OpenDialog.FileName := EditTable.Text;
  If OpenDialog.Execute Then
  Begin
    EditTable.Text := OpenDialog.FileName;
    Table.Close;
    DBConnectTo(DataBase, ExtractFileDir(OpenDialog.FileName));
    Table.TableName := ExtractFileName(OpenDialog.FileName);
    Try
      Table.Open;
      SetTableInfo;
    Except
    End;
    BtnVerify.Enabled := Table.Active;
    BtnRebuild.Enabled := Table.Active;
    BtnImpac.Enabled := Table.Active;
    BtnRegen.Enabled := Table.Active;
    Table.Close;
  End;
end;

end.
