unit UtilWrap;

interface

uses
  Windows, Messages, SysUtils, Classes, Dialogs, Bde, DbiTypes;

{ TUtility (TUVerifyTable) Session Options }
Const
  TU_Append_Errors = 1;
  TU_No_Secondary  = 2;
  TU_No_Warnings   = 4;
  TU_Header_Only   = 8;
  TU_Dialog_Hide   = 16;
  TU_No_Lock       = 32;

{ TUtility type definitions }
Type
  hTUses  = Word;
  phTUses = ^hTUses;

  { Verify Callback processes }
  TUVerifyProcess = (TUVerifyHeader, TUVerifyIndex, TUVerifyData, TUVerifySXHeader,
                     TUVerifySXIndex, TUVerifySXData, TUVerifySXIntegrity,
                     TUVerifyTableName, TURebuild);

  { Call back info for Verify Callback Function }
  TUVerifyCallBack = record
    PercentDone: word;
    TableName: DBIPath;
    Process: TUVerifyProcess;
    CurrentIndex: word;
    TotalIndex: word;
  end;

  { Functions Prototypes }
  TUInitFunc = Function(var hTUSession: hTUses): DBIResult; Stdcall;

  TUVerifyTableFunc = Function(hTUSession: hTUses;
                               pszTableName,
                               pszDriverType,
                               pszErrTableName,
                               pszPassword: PChar;
                               iOptions: integer;
                               var piErrorLevel: Integer): DBIResult; Stdcall;

  TURebuildTableFunc = Function(hTUSession: hTUses;
                                pszTableName,
                                pszDriverType,
                                pszBackupTableName,
                                pszKeyviolName,
                                pszProblemTableName: PChar;
                                pCrDesc: pCRTblDesc): DBIResult; Stdcall;

  TUGetCRTblDescCountFunc = Function(hTUSession: hTUses;
                                     pszTableName: PChar;
                                     var iFldCount,
                                     iIdxCount,
                                     iSecRecCount,
                                     iValChkCount,
                                     iRintCount,
                                     iOptParams,
                                     iOptDataLen: word): DBIResult; Stdcall;

  TUFillCRTblDescFunc = Function(hTUSession: hTUses;
                                 pCrDesc: pCRTblDesc;
                                 pszTableName,
                                 pszPassword: PChar): DBIResult; Stdcall;

  TUFillCURPropsFunc = Function(hTUSession: hTUses;
                                pszTableName: PChar;
                                var tblProps: CURProps): DBIResult; Stdcall;

  TUGetExtTblPropsFunc = Function(hTUSession: hTUses;
                                  pszTableName: PChar;
                                  var pTS: TimeStamp;
                                  var pbReadOnly: Boolean): DBIResult; Stdcall;

  TUExitFunc = Function(hTUSession: hTUses): DBIResult; Stdcall;

  TUGetErrorStringFunc = Function(iErrorcode: DBIResult;
                                  pszError: PChar): DBIResult; Stdcall;

  TUtilWrapper = Class
  Private
    UtilInstance: HMODULE;
  Public
    UInit:              TUInitFunc;
    UVerifyTable:       TUVerifyTableFunc;
    URebuildTable:      TURebuildTableFunc;
    UGetCRTblDescCount: TUGetCRTblDescCountFunc;
    UFillCRTblDesc:     TUFillCRTblDescFunc;
    UFillCURProps:      TUFillCURPropsFunc;
    UGetExtTblProps:    TUGetExtTblPropsFunc;
    UExit:              TUExitFunc;
    UGetErrorString:    TUGetErrorStringFunc;
    Constructor Create(DllPath: String);
    Destructor Destroy; Override;
  End;

implementation

Constructor TUtilWrapper.Create(DllPath: String);
Begin
  If Not FileExists(DllPath) Then
  Begin
    ShowMessage('Module not Found!');
    Exit;
  End;
  UtilInstance := LoadLibrary(PChar(DllPath));
  If UtilInstance <> 0 Then
  Begin
    @UInit              := GetProcAddress(UtilInstance, PChar('TUInit'));
    @UVerifyTable       := GetProcAddress(UtilInstance, PChar('TUVerifyTable'));
    @URebuildTable      := GetProcAddress(UtilInstance, PChar('TURebuildTable'));
    @UGetCRTblDescCount := GetProcAddress(UtilInstance, PChar('TUGetCRTblDescCount'));
    @UFillCRTblDesc     := GetProcAddress(UtilInstance, PChar('TUFillCRTblDesc'));
    @UFillCURProps      := GetProcAddress(UtilInstance, PChar('TUFillCURProps'));
    @UGetExtTblProps    := GetProcAddress(UtilInstance, PChar('TUGetExtTblProps'));
    @UExit              := GetProcAddress(UtilInstance, PChar('TUExit'));
    @UGetErrorString    := GetProcAddress(UtilInstance, PChar('TUGetErrorString'));
  End;
End;

Destructor TUtilWrapper.Destroy;
Begin
  If UtilInstance <> 0 Then
    FreeLibrary(UtilInstance);
  Inherited Destroy;
End;

end.
