unit PlugMan;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Extctrls, DBTables;

Type
  TPlugInLinker = Class;

  TPlugInForm = Class(TForm)
  Protected
    FLinker: TPlugInLinker;
  Public
    Procedure Init(Linker: TPlugInLinker); Virtual;
    Procedure AppNotify(Form: TForm; Action: String; Parms: Variant); Virtual;
    Procedure NotifyLinker(Action: String; Parms: Variant); Virtual;
  End;

  TPlugInfo = Class
  Public
    Index: Integer;
    Instance: HMODULE;
    Linker: TPlugInLinker;
    Path: String;
    Name: String;
    Desc: String;
    Info: String;
    Vers: String;
    Glyph: TBitmap;
    Started: Boolean;
  End;

  TPlugInfoList = Class(TList)
  Public
    Destructor Destroy; Override;
    Function AddInfo: TPlugInfo;
    Procedure Clear; Override;
    Function Find(Nam: String): TPlugInfo;
  End;

  TPlugManagerFormClose = Procedure(Sender: TObject; PlugIdx: Integer; Form: TPlugInForm) Of Object;
  TPlugManagerNotify = Procedure(Sender: TObject; Form: TPlugInForm; Action: String; Parms: Variant) Of Object;

  TPlugInManager = Class(TComponent)
  private
    FFolder: String;
    FExtension: String;
    PlugList: TPlugInfoList;
    FOnFormClose: TPlugManagerFormClose;
    FOnAppNotify: TPlugManagerNotify;
    Function GetCount: Integer;
    Function GetPlugInfo(Index: Integer): TPlugInfo;
  protected
    Function SlashSep(Path, S: String): String;
    Function GetFileList(Directory, FileMask: String; List: TStrings): Integer;
    Function GetPlugInVer(FileName: String): String;
    Procedure TerminatePlugin(Index: Integer);
    Procedure TerminatePlugins;
  public
    Constructor Create(AOwner : TComponent); Override;
    Destructor Destroy; Override;
    Procedure GetPluginInfo(FileName: String);
    Procedure UnLoadPlugin(Index: Integer);
    Procedure CollectPlugins;
    Procedure UnLoadPlugins;
    Procedure ClearPlugins;
    Procedure CloseAllForms;
    Function RunPlugIn(ID: Variant): TPlugInForm;
    Procedure FormClosed(Linker: TPlugInLinker; Form: TPlugInForm);
    Procedure NotifyPlugIns(Form: TForm; Action: String; Parms: Variant);
    Procedure NotifyApplication(Form: TPlugInForm; Action: String; Parms: Variant);
    Property Count: Integer Read GetCount;
    Property PlugInfo[Index: Integer]: TPlugInfo Read GetPlugInfo;
  published
    Property Folder: String Read FFolder Write FFolder;
    Property Extension: String Read FExtension Write FExtension;
    Property OnFormClose: TPlugManagerFormClose Read FOnFormClose Write FOnFormClose;
    Property OnAppNotify: TPlugManagerNotify Read FOnAppNotify Write FOnAppNotify;
  end;

  TFormCreateProc  = Procedure(HostApp: TApplication; HostSes: TSession; Var Form: TPlugInForm); StdCall;
  TFinalizeProc  = Procedure(BackApp: TApplication; BackSes: TSession); StdCall;

  TPlugInLinker = Class(TObject)
  private
    FHostApplic: TApplication;
    FHostSession: TSession;
    FBackApplic: TApplication;
    FBackSess: TSession;
    FCanUnload: Boolean;
    FManager: TPlugInManager;
    FFormStyle: TFormStyle;
    FIsModal: Boolean;
    FMultiple: Boolean;
    RunList: TList;
  public
    FormCreateProc: TFormCreateProc;
    FinalizeProc: TFinalizeProc;
    Constructor Create;
    Destructor Destroy; Override;
    Procedure Init(PlugMan: TPlugInManager; HostApp: TApplication; HostSes: TSession);
    Function Run: TPlugInForm;
    Procedure CloseAll;
    Procedure Terminate;
    Procedure FormClosed(Form: TPlugInForm);
    Procedure NotifyForms(Form: TForm; Action: String; Parms: Variant);
    Procedure NotifyPlugMan(Form: TPlugInForm; Action: String; Parms: Variant);
    Property FormStyle: TFormStyle Read FFormStyle Write FFormStyle;
    Property IsModal: Boolean Read FIsModal Write FIsModal;
    Property Multiple: Boolean Read FMultiple Write FMultiple;
    Property BackApplic: TApplication Read FBackApplic Write FBackApplic;
    Property BackSess: TSession Read FBackSess Write FBackSess;
    Property CanUnload: Boolean Read FCanUnload Write FCanUnload;
  end;

//procedure Register;

implementation

{procedure Register;
begin
  RegisterComponents('Selesta', [TPlugInManager]);
end;}

// *************************** TPlugInfoList **********************************


Destructor TPlugInfoList.Destroy;
Begin
  Clear;
  Inherited Destroy;
End;

Function TPlugInfoList.AddInfo: TPlugInfo;
Begin
  Result := TPlugInfo.Create;
  Result.Glyph := TBitmap.Create;
  Result.Index := Add(Result);
  Result.Instance := 0;
  Result.Linker := Nil;
  Result.Started := False;
End;

Procedure TPlugInfoList.Clear;
Var
  I: Integer;
  Item: TPlugInfo;
Begin
  For I := 0 To Count-1 Do
  Begin
    Item := TPlugInfo(Items[I]);
    Item.Glyph.Free;
    Item.Free;
  End;
  Inherited Clear;
End;

Function TPlugInfoList.Find(Nam: String): TPlugInfo;
Var
  I: Integer;
Begin
  Result := Nil;
  For I := 0 To Count-1 Do
    If UpperCase(TPlugInfo(Items[I]).Name) = UpperCase(Nam) Then
    Begin
      Result := TPlugInfo(Items[I]);
      Break;
    End;
End;


// *************************** TPlugInManager **********************************

Constructor TPlugInManager.Create(AOwner : TComponent);
Begin
  Inherited Create(AOwner);
  PlugList := TPlugInfoList.Create;
  FFolder := '';
  FExtension := 'DLL';
  FOnFormClose := Nil;
  FOnAppNotify := Nil;
End;

Destructor TPlugInManager.Destroy;
Begin
  TerminatePlugins;
  PlugList.Free;
  Inherited Destroy;
End;

Function TPlugInManager.GetCount: Integer;
Begin
  Result := PlugList.Count;
End;

Function TPlugInManager.GetPlugInfo(Index: Integer): TPlugInfo;
Begin
  Result := Nil;
  If (Index >= 0) And (Index < PlugList.Count) Then
    Result := TPlugInfo(PlugList[Index]);
End;

Function TPlugInManager.SlashSep(Path, S: String): String;
Const
  Slashes: array [False..True] of PChar = ('','\');
Begin
  If Path = '' Then Result := ''
  Else Result := Format('%s%s%s',[Path, Slashes[Path[Length(Path)] <> '\'], S]);
End;

Function TPlugInManager.GetFileList(Directory, FileMask: String; List: TStrings): Integer;
var
  FileInfo: TWin32FindData;
  FindFileH: THandle;
  Dir, FullName: String;
  FName: String;
Begin
  Result := 0;
  If List = Nil Then Exit;
  List.Clear;
  Dir := SlashSep(Directory, FileMask);
  FindFileH := FindFirstFile(PChar(Dir), FileInfo);
  If FindFileH <> INVALID_HANDLE_VALUE then
  Begin
    Repeat
      With FileInfo Do
        If (dwFileAttributes And FILE_ATTRIBUTE_DIRECTORY) = 0 Then
        Begin
          FName := StrPas(cFileName);
          If (FName <> '.') And (FName <> '..') Then
          Begin
            FullName := UpperCase(SlashSep(Directory, FName));
            List.Add(FullName);
          End;
        End;
    Until Not FindNextFile(FindFileH, FileInfo);
    Windows.FindClose(FindFileH);
  End;
  Result := List.Count;
End;

Function TPlugInManager.GetPlugInVer(FileName: String): String;
Var
  Buffer: Array[0..4096] Of Byte;
  VerInf: Pointer;
  VerLen: DWORD;
  VM, VL: DWORD;
Begin
  Result := '';
  Try
    If GetFileVersionInfo(PChar(FileName), 0, 4096, @Buffer) Then
      If VerQueryValue(@Buffer, '\', VerInf, VerLen) Then
      Begin
        VM := PVSFixedFileInfo(VerInf).dwFileVersionMS;
        VL := PVSFixedFileInfo(VerInf).dwFileVersionLS;
        Result := IntToStr(HIWORD(VM)) + '.' +
                  IntToStr(LOWORD(VM)) + '.' +
                  IntToStr(HIWORD(VL)) + '.' +
                  IntToStr(LOWORD(VL));
      End;
  Except
  End;
End;

Procedure TPlugInManager.TerminatePlugin(Index: Integer);
Type
  TPlugInStopProc = Procedure; StdCall;
Var
  Info: TPlugInfo;
  StopProc: TPlugInStopProc;
  CanU: Boolean;
Begin
  CanU := False;
  Info := PlugInfo[Index];
  If Assigned(Info) Then
    If Info.Started Then
    Begin
      Try
        CanU := Info.Linker.CanUnload;
        Info.Linker.Terminate;
        Application.ProcessMessages;
      Except
      End;
      If Info.Instance <> 0 Then
      Begin
        @StopProc := GetProcAddress(Info.Instance, PChar('StopPlugIn'));
        If @StopProc <> Nil Then
        Try
          StopProc;
          If CanU Then
          Begin
            FreeLibrary(Info.Instance);
            Info.Instance := 0;
            Info.Started := False;
          End;
        Except
        End;
      End;
    End;
End;

Procedure TPlugInManager.TerminatePlugins;
Var
  I: Integer;
Begin
  For I := 0 To PlugList.Count-1 Do
    TerminatePlugIn(I);
  PlugList.Clear;
End;

Procedure TPlugInManager.GetPluginInfo(FileName: String);
Type
  TPlugInInfoProc = Procedure(Var N, D, I: String); StdCall;
Var
  PlugInInstance: HMODULE;
  HResource: HRSRC;
  InfoProc: TPlugInInfoProc;
  N, D, I: String;
  Item: TPlugInfo;
Begin
  PlugInInstance := LoadLibrary(PChar(FileName));
  If PlugInInstance <> 0 Then
  Begin
    @InfoProc := GetProcAddress(PlugInInstance, PChar('GetPlugInInfo'));
    If @InfoProc <> Nil Then
    Begin
      Item := PlugList.AddInfo;
      Item.Path := FileName;
      Try
        InfoProc(N, D, I);
        Item.Name := N;
        Item.Desc := D;
        Item.Info := I;
      Except
        Item.Name := ExtractFileName(FileName);
        Item.Desc := '';
        Item.Info := '';
      End;
      Item.Vers := GetPlugInVer(FileName);
      HResource := FindResource(PlugInInstance, PChar('PLUGINMENUIMAGE'), RT_BITMAP);
      If HResource <> 0 Then
        Item.Glyph.LoadFromResourceName(PlugInInstance, 'PLUGINMENUIMAGE')
      Else
      Begin
        Item.Glyph.Width := 16;
        Item.Glyph.Height := 16;
      End;
    End;
    FreeLibrary(PlugInInstance);
  End;
End;

Procedure TPlugInManager.UnLoadPlugin(Index: Integer);
Type
  TPlugInStopProc = Procedure; StdCall;
Var
  Info: TPlugInfo;
  StopProc: TPlugInStopProc;
Begin
  Info := PlugInfo[Index];
  If Assigned(Info) Then
    If Info.Started Then
      If Info.Linker.CanUnload Then
      Begin
        Try
          Info.Linker.Terminate;
          Application.ProcessMessages;
        Except
        End;
        If Info.Instance <> 0 Then
        Begin
          @StopProc := GetProcAddress(Info.Instance, PChar('StopPlugIn'));
          If @StopProc <> Nil Then
          Try
            StopProc;
            FreeLibrary(Info.Instance);
            Info.Instance := 0;
            Info.Started := False;
          Except
          End;
        End;
      End;
End;

Procedure TPlugInManager.CollectPlugins;
Var
  Path: String;
  FileList: TStringList;
  I: Integer;
Begin
  FileList := TStringList.Create;
  Path := ExtractFilePath(ParamStr(0));
  If FFolder <> '' Then
    Path := SlashSep(Path, FFolder);
  GetFileList(Path, '*.'+FExtension, FileList);
  For I := 0 To FileList.Count-1 Do
    GetPluginInfo(FileList[I]);
  FileList.Free;
End;

Procedure TPlugInManager.UnLoadPlugins;
Var
  I: Integer;
Begin
  For I := 0 To PlugList.Count-1 Do
    UnloadPlugIn(I);
End;

Procedure TPlugInManager.ClearPlugins;
Begin
  UnLoadPlugins;
  PlugList.Clear;
End;

Procedure TPlugInManager.CloseAllForms;
Var
  I: Integer;
  Info: TPlugInfo;
Begin
  For I := 0 To PlugList.Count-1 Do
  Begin
    Info := PlugInfo[I];
    If Assigned(Info) Then
      If Info.Started Then
      Begin
        Try
          Info.Linker.CloseAll;
          Application.ProcessMessages;
        Except
        End;
      End;
  End;
End;

Function TPlugInManager.RunPlugIn(ID: Variant): TPlugInForm;
Type
  TPlugInStartProc = Function: TPlugInLinker; StdCall;
Var
  StartProc: TPlugInStartProc;
  Item: TPlugInfo;
Begin
  Result := Nil;
  Item := Nil;
  If VarType(ID) = varString Then
    Item := PlugList.Find(ID)
  Else If VarType(ID) = varInteger Then
    Item := PlugInfo[ID];
  If Item = Nil Then Exit;

  If Item.Started Then
  Begin
    If Item.Linker <> Nil Then
      Result := Item.Linker.Run;
  End
  Else
  Begin
    Item.Instance := LoadLibrary(PChar(Item.Path));
    If Item.Instance <> 0 Then
    Begin
      @StartProc := GetProcAddress(Item.Instance, PChar('StartPlugIn'));
      If @StartProc <> Nil Then
      Try
        Item.Linker := StartProc;
        Item.Linker.Init(Self, Application, Session);
        Item.Started := True;
        Result := Item.Linker.Run;
        If Not Assigned(Result) Then
          UnLoadPlugin(Item.Index);
      Except
      End;
    End;
  End;
End;

Procedure TPlugInManager.FormClosed(Linker: TPlugInLinker; Form: TPlugInForm);
Var
  I, Idx: Integer;
Begin
  Idx := -1;
  For I := 0 To PlugList.Count-1 Do
    If TPlugInfo(PlugList[I]).Linker = Linker Then
    Begin
      Idx := I;
      Break;
    End;
  If Assigned(FOnFormClose) Then FOnFormClose(Self, Idx, Form);
End;

Procedure TPlugInManager.NotifyPlugIns(Form: TForm; Action: String; Parms: Variant);
Var
  I: Integer;
  Info: TPlugInfo;
Begin
  For I := 0 To PlugList.Count-1 Do
  Begin
    Info := PlugInfo[I];
    If Assigned(Info) Then
      If Info.Started Then
      Begin
        Info.Linker.NotifyForms(Form, Action, Parms);
      End;
  End;
End;

Procedure TPlugInManager.NotifyApplication(Form: TPlugInForm; Action: String; Parms: Variant);
Begin
  If Assigned(FOnAppNotify) Then
    FOnAppNotify(Self, Form, Action, Parms);
End;


// **************************** TPlugInLinker **********************************

Constructor TPlugInLinker.Create;
Begin
  Inherited Create;
  FHostApplic := Nil;
  FBackApplic := Nil;
  FBackSess := Nil;
  FCanUnload := True;
  FManager := Nil;
  FFormStyle := fsNormal;
  FIsModal := True;
  FMultiple := False;
  @FormCreateProc := Nil;
  @FinalizeProc := Nil;
  RunList := TList.Create;
End;

Destructor TPlugInLinker.Destroy;
Begin
  RunList.Free;
  Inherited Destroy;
End;

Procedure TPlugInLinker.Init(PlugMan: TPlugInManager; HostApp: TApplication; HostSes: TSession);
Begin
  FManager := PlugMan;
  FHostApplic := HostApp;
  FHostSession := HostSes;
End;

Function TPlugInLinker.Run: TPlugInForm;
Var
  Form: TPlugInForm;
Begin
  Form := Nil;
  If RunList.Count > 0 Then
    Form := TPlugInForm(RunList[0]);

  If (Form = Nil) Or FMultiple Then
    If @FormCreateProc <> Nil Then
    Begin
      FormCreateProc(FHostApplic, FHostSession, Form);
      If Assigned(Form) Then
      Begin
        Form.Init(Self);
        If FIsModal Then
        Begin
          Form.Close;
          Form.ShowModal;
          Form.Free;
          Form := Nil;
        End
        Else
        Begin
          RunList.Add(Form);
          If FMultiple Then
            Form.Caption := Form.Caption + ' #'+IntToStr(RunList.Count);
        End;
      End;
    End;

  If (Form <> Nil) Then
    If Not Form.Showing Then Form.Show;
  Result := Form;
End;

Procedure TPlugInLinker.CloseAll;
Var
  I: Integer;
  Form: TPlugInForm;
Begin
  For I := RunList.Count-1 DownTo 0 Do
  Try
    Form := TPlugInForm(RunList[I]);
    Form.Close;
    Application.ProcessMessages;
  Except
  End;
  RunList.Clear;
End;

Procedure TPlugInLinker.Terminate;
Begin
  CloseAll;
  If @FinalizeProc <> Nil Then
    FinalizeProc(FBackApplic, FBackSess);
End;

Procedure TPlugInLinker.FormClosed(Form: TPlugInForm);
Var
  I: Integer;
Begin
  For I := RunList.Count-1 DownTo 0 Do
    If TPlugInForm(RunList[I]) = Form Then
    Begin
      RunList.Delete(I);
      Break;
    End;
  FManager.FormClosed(Self, Form);
End;

Procedure TPlugInLinker.NotifyForms(Form: TForm; Action: String; Parms: Variant);
Var
  I: Integer;
Begin
  For I := RunList.Count-1 DownTo 0 Do
  Begin
    TPlugInForm(RunList[I]).AppNotify(Form, Action, Parms);
  End;
End;

Procedure TPlugInLinker.NotifyPlugMan(Form: TPlugInForm; Action: String; Parms: Variant);
Begin
  FManager.NotifyApplication(Form, Action, Parms);
End;


// **************************** TPlugInLinker **********************************

Procedure TPlugInForm.Init(Linker: TPlugInLinker);
Begin
  FLinker := Linker;
  // (Redefine if you need to do somethings at start time)
End;

Procedure TPlugInForm.AppNotify(Form: TForm; Action: String; Parms: Variant);
Begin
  // Dummy Procedure (Redefine if necessary)
End;

Procedure TPlugInForm.NotifyLinker(Action: String; Parms: Variant);
Begin
  If Assigned(FLinker) Then
    FLinker.NotifyPlugMan(Self, Action, Parms);
End;

end.
